		/* PROCEDURE TO CONSTRUCT HIT AND MISS TABLES FROM PROBABILITIES */ 
		/* NOTE ALSO CONTAINS PROCEDURES FOR VARIOUS SIMULATED HIT AND MISS MODELS */
/*
**     Procedure to calculate Hit and Miss Tables		**                                                      
**     takes form                                                       	**                                    
**     { table } = hitmiss(pij,jvec);			       		**                         
**     Inputs: pij  - n by J matrix of probabilities		**  
**             jvec - n by 1 vector of observed choices	**
**     Output: table - hit and miss table, coded j=0,...,J-1**
*/


proc 1 = hitmiss(pij,jvec);
local capj, n, jrep, d_ij, pmax, table, jrepi, seld_ij, check;
local topa, fmta, maska, maskb, fmtb, topb, fmtb1, fmtb2, fmtb3, middle, maskc, fmtc1, fmtc2, fmtc;
local bottom, maskd, fmtd1, fmtd2, fmtd, a, b, c, d, equal, pmaxr;
local spacea;


/* DEFINE # OF CHOICES & SAMPLE SIZE */

if minc(jvec) eq 1;
	jvec = jvec - 1;
endif;
capJ	= maxc(jvec) + 1;
n	= rows(jvec);


/* SET UP MATRIX OF ZEROS AND ONES DEFINING ALTERNATIVE CHOSEN */

jrep 	= 0;
d_ij	= zeros(n,1);
do while jrep lt capJ;
	d_ij = d_ij~(jvec .eq jrep);
jrep=jrep+1;
endo;

d_ij 	= d_ij[.,2:capJ+1];


/* CHECK PROBABILITIES SUM TO 1 */

check = meanc(sumc(pij'));

if check gt 1.01 or check lt (1-0.01);
	errorlog "PROBABILITIES DON'T SUM TO 1: AVERAGE PROB IS";check;stop;
endif;

/* DEFINE MATRIX OF MAXIMUM PROBABLILITIES */

pmax 	= pij .eq (maxc(pij') .*. ones(1,capJ)); 
table	= zeros(capj,capj);

if sumc(sumc(d_ij')) ne n;
    "error in Y's";stop;
endif;

equal = sumc(pmax') .gt 1;

pmaxr = recode(pmax[.,1:capJ-1],equal,0);

pmax    = pmaxr~pmax[.,capJ];

if sumc(sumc(pmax')) ne n;
    "error in p_ij's - hitmiss.prc, line 63";stop;
endif;


jrep 	= 1;
do while jrep le capj;
	jrepi=1;
	table[jrep,.] = (sumc(selif(pmax,d_ij[.,jrep] .eq 1)))'; 
	jrep 	= jrep + 1;
endo;

table = table|(sumc(table)');
table = table~(sumc(table'));

if table[rows(table),cols(table)] ne n;
	errorlog "error in suming elements of hit-miss table";
	table;?;n;stop;
endif;

let topa[1,2]   = " " "Pred";
let spacea      = " ";
spacea          = spacea * ones(1,capJ);
topa            = topa~spacea;

maska 			= zeros(1,capJ+2);
fmta			= " *.*lf"~8~8;

topb			= "Actual"~(seqa(0,1,capJ)')~"Total"; 
maskb			= ones(1,capJ+2);		
maskb[1]		= 0;
maskb[capJ+2] 	= 0;

fmtb1			= "-*.*s"~8~8;
fmtb3			= "*.*lf"~8~8;
fmtb2			= ones(capJ,1) .*. ("*.*lf  "~8~0);
fmtb				= fmtb1|fmtb2|fmtb3;

middle			= seqa(0,1,capJ)~table[1:rows(table)-1,.1];
maskc			= ones(1,capJ+2);
fmtc				= ones(capJ+2,1) .*. ("*.*lf  "~8~0);

bottom 			= "Total"~table[rows(table),.];
maskd			= 0~ones(1,capJ+1);
fmtd1			= "-*.*s"~8~8;
fmtd2			= ones(capJ+1,1) .*. ("*.*lf "~8~0);
fmtd				= fmtd1|fmtd2;

print;
" HIT AND MISS TABLE";PRINT;
a=printfm(topa,maska,fmta);" ";
a=printfm(topb,maskb,fmtb);" ";
a=printfm(middle,maskc,fmtc);
a=printfm(bottom,maskd,fmtd);

retp(table);
endp;

		/* PROCEDURES FOR RANDOM DRAWS FOR SIMULATED TABLES */


proc _Probit(xbeta,n,J,capI);		@ STANDARD PROBIT NORMALS @
local eij, UMat, pU, UMax;

	if capI eq 1;
		eij = zeros(n,J-1);
	else;
		eij = rndn(n,J-1);	
	endif;

	UMat 	= (xbeta) + eij;	
  	pU    	= (1-cdfn(Umat))~cdfn(UMat);
   	UMax  	= (maxc(UMat') .*. ones(1,cols(J))) .eq UMat;

retp( pU );
endp;


proc _Probhet(xbeta,zgamma,n,capI);		@ STANDARD PROBIT NORMALS @
local eij, UMat, pU, UMax, ystar, m;

	if capI le 1;
        UMat 	= (xbeta)./zgamma;	   
  	    pU    	= (1-cdfn(Umat))~cdfn(UMat);
	else;
       eij     = rndn(n,capI) .* exp(zgamma);		   
       UMat    = (xbeta) + eij;		
       ystar   = UMat .gt 0;
       pU      = meanc(ystar');                 
       pU      = (1-pU)~pU;  
	endif;


    UMax  	= (maxc(UMat') .*. ones(1,cols(2))) .eq UMat;

retp( pU );
endp;


proc _PProbit(xbeta,sigma,n,nt,capI,idvec);		@ PANEL PROBIT NORMALS @
local eij, ai, UMat, pU, UMax, irep, sel;

	if capI eq 1;
		eij 	= zeros(nt,1);
		ai	= zeros(nt,1);
	else;
		eij 	= rndn(nt,1);	
		ai	= 0;

		for irep(1,n,1);
			sel = sumc(idvec .eq irep);
			ai = ai|
				((rndn(1,1)*sigma) .*. ones(sel,1));
		endfor;

		ai	= ai[2:rows(ai)];

	endif;

	UMat 	= (xbeta) + eij + ai;	
  	pU    	= (1-cdfn(Umat))~cdfn(UMat);
    	@ UMax  	= (maxc(UMat') .*. ones(1,cols(J))) .eq UMat; @
    	UMax  	= (maxc(UMat') .*. ones(1,1)) .eq UMat;

retp( pU );
endp;


proc _Logit(xbeta,n,J,capI);		@ EXTREME VALUE - MULTINOMIAL LOGIT @
local eij, UMat, v_ij, pU, UMax;

	if capI eq 1;
		eij = zeros(n,J);	
	else;
		eij = -ln(-ln(rndu(n,J)));
	endif;

	UMat 	= xbeta + eij;	
	v_ij	= exp(UMat); 
	pU		= v_ij ./ sumc(v_ij');

retp( pU );
endp;

proc _OProbit(xbeta,mu,n,J,capI);
local eij, pU, jrep2; 

	if capI eq 1;
		eij 	= zeros(n,1);	
	else;
		eij 	= rndn(n,1);
	endif;

	xbeta = xbeta + eij;

	pU	= 1-(cdfn(xbeta));
	pU	= pU~(cdfn(mu[1]-xbeta) - cdfn(-xbeta));
	jrep2= 2;
	do while jrep2 le J-2;
		pU = pU~(cdfn(mu[jrep2]-xbeta) -
          	            	cdfn(mu[jrep2-1]-xbeta));
		jrep2 = jrep2 + 1;
	endo;

	pU = pU~(1 - cdfn(mu[J-2]-xbeta));

retp( pU );
endp;


proc _NegBin(xbeta,alpha,n,J,capI,range,y); 
local expeij, eij, UMat, v_ij, pU, UMax, mu_i, m, rangefac, tempU, theta, pois;
local lambdai, lnpU, lnpUtmp, temp, pUtmp; 

	mu_i= exp(xbeta);
	theta = 1/alpha;	
	
	if capI le 1;	
		lnpU = theta .* ln(theta ./ (theta + mu_i));

		for m (2,rows(range),1);	
			lnpUtmp = ln(mu_i ./ (theta + mu_i)) + ln(theta + (m-2)) - ln(m-1) + lnpU[.,m-1];
			lnpU = lnpU~lnpUtmp;
		endfor;

		pU = exp(lnpU);		

	/*
		mu_i=round(mu_i);		
		pUtmp = (recode(mu_i,mu_i .ge maxc(range),maxc(range))) .*. ones(1,J);

		pU = pUtmp .eq (range' .*. ones(n,1));
	*/
	
	else;

	@	{expeij} = rndgama(n, 1, alpha, theta, 0); @

		expeij = rndgam(n,1,alpha) + ( 1 - alpha);
	
@expeij = ones(n,1);  @

		eij = ln(expeij); 

		mu_i= exp(xbeta + eij);

/*				
		lnpU = theta .* ln(theta ./ (theta + mu_i));

		for m (2,rows(range),1);	
			lnpUtmp = ln(mu_i ./ (theta + mu_i)) + ln(theta + (m-2)) - ln(m-1) + lnpU[.,m-1];
			lnpU = lnpU~lnpUtmp;
		endfor;

		pU = exp(lnpU);		
*/

		pois = rndp(n,1,mu_i); 

		pois = recode(pois,pois .ge maxc(range),maxc(range));
		pU 	= pois .eq (range' .*. ones(n,1));


	endif;

retp( pU );
endp;


proc _Ogev(xbeta,rho,n,capJ,range,y);
local thetaall, jj, i, v_ij, v_ijall, p_denom, p_a, p_b, p_ogev, pU, mark;

	thetaall = zeros(capJ,1);

	v_ij		= exp(xbeta ./ rho); 
	v_ijall	= zeros(n,1)~v_ij~zeros(n,1);

	p_denom	= (v_ijall[.,1:capJ+1] + v_ijall[.,2:capJ+2]).^rho;

	p_a		= (v_ij ./ sumc(p_denom'));  

	p_b		= ((v_ijall[.,1:capJ] + v_ij) .^ (rho-1)) + 
			((v_ij + v_ijall[.,3:capJ+2]) .^ (rho-1)); 
			
	p_ogev	= p_a .* p_b;

	pU = (thetaall' ./ (sumc(thetaall) + 1)) + ((1./ (sumc(thetaall) + 1)) .* p_ogev);

retp( pU );
endp;

proc _Dogit(xbeta,theta,thetas,n,capJ,range,y);
local thetaall, jj, i, v_ij, v_ijall, p_denom, p_a, p_b, p_ogev, pU, rho;

	rho		=1;	
	thetaall	=0;
	jj		=1;
	i		=1;
	do while i le capJ;
		if thetas[i] eq 1;
			thetaall	= thetaall|theta[jj];
			jj		= jj+1;
		else;
			thetaall	= thetaall|0;
		endif;
	i=i+1;
	endo;

	thetaall=thetaall[2:rows(thetaall)];

	v_ij		= exp(xbeta ./ rho); 
	v_ijall	= zeros(n,1)~v_ij~zeros(n,1);

	p_denom	= (v_ijall[.,1:capJ+1] + v_ijall[.,2:capJ+2]).^rho;

	p_a		= (v_ij ./ sumc(p_denom'));  

	p_b		= ((v_ijall[.,1:capJ] + v_ij) .^ (rho-1)) + 
			((v_ij + v_ijall[.,3:capJ+2]) .^ (rho-1)); 
			
	p_ogev	= p_a .* p_b;

	pU = (thetaall' ./ (sumc(thetaall) + 1)) + ((1./ (sumc(thetaall) + 1)) .* p_ogev);

retp( pU );
endp;



proc _Dogev(xbeta,theta,thetas,rho,n,capJ,range,y);
local thetaall, jj, i, v_ij, v_ijall, p_denom, p_a, p_b, p_ogev, pU;

	thetaall	=0;
	jj		=1;
	i		=1;
	do while i le capJ;
		if thetas[i] eq 1;
			thetaall	= thetaall|theta[jj];
			jj		= jj+1;
		else;
			thetaall	= thetaall|0;
		endif;
	i=i+1;
	endo;

	thetaall=thetaall[2:rows(thetaall)];

	v_ij		= exp(xbeta ./ rho); 
	v_ijall	= zeros(n,1)~v_ij~zeros(n,1);

	p_denom	= (v_ijall[.,1:capJ+1] + v_ijall[.,2:capJ+2]).^rho;

	p_a		= (v_ij ./ sumc(p_denom'));  

	p_b		= ((v_ijall[.,1:capJ] + v_ij) .^ (rho-1)) + 
			((v_ij + v_ijall[.,3:capJ+2]) .^ (rho-1)); 
			
	p_ogev	= p_a .* p_b;

	pU = (thetaall' ./ (sumc(thetaall) + 1)) + ((1./ (sumc(thetaall) + 1)) .* p_ogev);

retp( pU );
endp;




/* PDFGAM - Standardized Gamma Probability Density function
**
** Usage:  d = pdfgam(x,a)
**
** Input:  X - matrix of Gamma values
**         A - matrix of shape parameter for Gamma distribution
**
** Output: D - matrix of density of Gamma(a) function at X
*/

proc 1 = pdfgam(x,a) ;
local gamxsd;
     if not(a > 0) ;
        "ERROR: PDFGAM - Shape parameter is not positive" ;
        gamxsd = -999;
	goto retry3;
	/*retp(-999) ;*/
     endif ;
    if x < 100 ;
	gamxsd = x^(a-1) .* exp(-x) ./ gamma(a);	
    else ;
	gamxsd = exp((a-1).*ln(x) - x - lnfact(a-1));
    endif ;
    retry3:
    retp(gamxsd) ;
endp ;


/* INVCDFG - Inverse of the Gamma Cumulative Distribution function
**           with shape parameter A
**
** Usage: x = invcdfg(p,a)
**
** Input:  P  - matrix of percentage points ( 0 < p < 1 )
**         A  - matrix of shape parameters (conformable with p)
**
** Output: X  - matrix of critical values st Pr(x < X) = P and x ~ Gamma(A)
*/

proc 1 = invcdfg(p,a) ;
     local converge,negative,tol,tol2,x0,x1,f0,df0,kg;
     if not(p >= 0 and p <= 1) ;
        "ERROR: INVCDFG - P not in range (0,1): Redraw" ;
        x0 = -999;
	goto retry;
	/* retp(-999); */
     endif ;
     if not(a > 0) ;
        "ERROR: INVCDFG - A is not positive: Redraw" ;
        x0 = -999;
	goto retry;
	/* retp(-999); */
     endif ;
     tol  = 1e-8 ;
     tol2 = 1e-20 ;
     x0   = a.*ones(rows(p),cols(p)) ;
     clear converge,kg ;
     do until( converge or kg > 50 ) ;
        f0  = cdfgam(a,x0) ;
        df0 = pdfgam(x0,a) ;
	if df0 eq -999;
		x0 = -999;
	goto retry;
	/* retp(-999); */
	endif;
	if f0 eq -999;
		x0 = -999;
	goto retry;
	/* retp(-999); */
	endif;
        if not(df0 > tol2) ;
           df0 = df0 + (df0 .< tol2).*(tol2 - df0) ;
        endif;
        x1  = x0 + (p - f0)./df0 ;
        negative = not(x1 > tol) ;
        if negative ;
           x1 = x1 + (x1.<=tol).*(x0.*(0.5 + 1.5.*(p.> f0)) - x1) ;
        endif ;
        converge = abs(x0 - x1) < tol and not negative ;
        x0 = x1 ;
        kg = kg + 1 ;
     endo ;
     if not converge ;
        /* "Warning: INVCDFG has not converged: Redraw" ; */
	x0 = -999;
	goto retry;
	/* retp(-999); */
     endif ;
     ndpclex ;
     retry:	
     retp(x0) ;
endp ;

/* RNDGAMA - Random numbers from a General Gamma Distribution
**
** Usage:  {x} = rndgama(nr,nc,a,b,c)
**
** Input:  NR - scalar of row size returned matrix
**         NC - scalar of column size returned matrix
**         A  - matrix of shape parameters for Gamma distribution
**         B  - matrix of scale parameters for Gamma distribution
**         C  - matrix of location parameters for Gamma distribution
**             (ExE conformable with RxC matrix)
**
** Output: x - R X C matrix of random numbers ~ Gamma(a,b,c) - if -999, error code
*/

proc 1 = rndgama(nr,nc,a,b,c);
local gamx,invgam;
	if not(a > 0) ;
        	"ERROR: RNDG - Shape parameter, A, is not positive: Redraw" ;
		/* retp(-999) ; */
		clear gamx;
		gamx = -999;
		goto retry2;
	endif ;
	if b eq -999 or c eq -999 or nr eq -999 or nc eq -999;
		/* retp(-999); */
		clear gamx;
		gamx = -999;
		goto retry2;
	endif;
	invgam = invcdfg(rndu(nr,nc),a);
	if sumc(vec(invgam) .eq -999) ge 1;
		/* retp(-999); */
		clear gamx;
		gamx = -999;
		goto retry2;
	endif;
	gamx = b.*invgam + c;
	retry2:
	retp(gamx) ;
endp ;



/* CDFGAMC - Complement of Gamma Cumulative Distribution function
**
** Format: p = cdfgamc(x,a)
**
** Input:  X - R x C matrix of gamma values
**         A - matrix of shape parameters
**             (must be E x E conformable with X)
**
** Output: P - R x C matrix of probabilities st. 1 - Pr(x <= X) = P
**             where x ~ Gamma(a)
*/

proc 1 = cdfgamc(x,a) ;
     retp(1 - cdfgam(a,x)) ;
endp ;

